import App
import loadspacehelper
import AI.Player.FlyForward
from Races import Races
from LibQBautostart import *


class Construct:
        def __init__(self, pConstructingShip, sConstructingRace, lShipsToConstruct, sConstructionLocation, iIncConstructionPower, iNumParallelConstructions = 1):
                self.iConstructingShipID = pConstructingShip.GetObjID()
                self.sConstructingRace = sConstructingRace
                self.lShipsToConstruct = lShipsToConstruct
                self.iBuildShipID = 0
                self.sConstructionLocation = sConstructionLocation
                self.iIncConstructionPower = iIncConstructionPower
                self.iNumParallelConstructions = iNumParallelConstructions
                self.iCurShipStatus = 0
                
                self.pTimerProcess = None
                self.SetupTimer()

        def GetConstructingShip(self):
                return App.ShipClass_GetObjectByID(None, self.iConstructingShipID)
        
        def GetBuildShip(self):
                return App.ShipClass_GetObjectByID(None, self.iBuildShipID)

        def SetupTimer(self):
                if self.pTimerProcess:
                        # We already have a timer.
                        return

		self.pTimerProcess = App.PythonMethodProcess()
		self.pTimerProcess.SetInstance(self)
		self.pTimerProcess.SetFunction("Update")
		self.pTimerProcess.SetDelay(1.0)
		self.pTimerProcess.SetPriority(App.TimeSliceProcess.LOW)

        def Update(self, dTimeAvailable):
                # check if the Object doesn't exist anymore
                if not self.GetConstructingShip():
                        # delete our timer
                        self.pTimerProcess = None
                        # remove our build ship if it exits
                        if self.GetBuildShip():
                                self.GetBuildShip().DestroySystem(self.GetBuildShip().GetHull())
                # only do anything if our Hull is > 50%
                elif self.GetConstructingShip().GetHull().GetConditionPercentage() > 0.5:
                        self.ContinueConstruction()
        
        def ContinueConstruction(self):
                if not self.GetBuildShip():
                        self.iCurShipStatus = 0
                
                if self.iCurShipStatus == 0:
                        self.CreateNewShip()
                elif self.iCurShipStatus == 1:
                        self.SetShipSystems(iIncC = self.iIncConstructionPower)
                elif self.iCurShipStatus == 2:
                        self.ShipLeaveDock()
                        self.iCurShipStatus = 3
                elif self.iCurShipStatus == 30:
                        self.ShipFinished()
                        self.iCurShipStatus = 31
                elif self.iCurShipStatus == 60:
                        self.iCurShipStatus = 0
                else:
                        self.iCurShipStatus = self.iCurShipStatus + 1
        
        def CreateNewShip(self):
                sShipType = self.GetRandomShip()
                if sShipType:
                        pSet = self.GetConstructingShip().GetContainingSet()
                        sShipName = Races[self.sConstructingRace].GetRandomName()
                        pNewShip = loadspacehelper.CreateShip(sShipType, pSet, sShipName, self.sConstructionLocation)
                        if not pNewShip:
                                return
                        self.GetConstructingShip().EnableCollisionsWith(pNewShip, 0)
                        self.iBuildShipID = pNewShip.GetObjID()
                        self.iCurShipStatus = 1
                        self.SetShipSystems(iToValue = 0.00000001)
                        pNewShip.DisableGlowAlphaMaps()
                        # Make it stationary, so it doesn't move on gune fire
                        pNewShip.GetShipProperty().SetStationary(1)
        
        def GetRandomShip(self):
                iRandTypeNum = App.g_kSystemWrapper.GetRandomNumber(len(self.lShipsToConstruct))
                return self.lShipsToConstruct[iRandTypeNum]

                
        def SetShipSystems(self, iToValue=0, iIncC=0):
                pShip = self.GetBuildShip()
                if not pShip:
                        self.iCurShipStatus = 0
                        return
                # disable glow every time, to make sure it hasn't been enabled
                pShip.DisableGlowAlphaMaps()
                pPropSet = pShip.GetPropertySet()
                pShipSubSystemPropInstanceList = pPropSet.GetPropertiesByType(App.CT_SUBSYSTEM_PROPERTY)
                iNumItems = pShipSubSystemPropInstanceList.TGGetNumItems()
                iNumConstructs = 0

                pShipSubSystemPropInstanceList.TGBeginIteration()
                for i in range(iNumItems):
                        pInstance = pShipSubSystemPropInstanceList.TGGetNext()
                        pProperty = pShip.GetSubsystemByProperty(App.SubsystemProperty_Cast(pInstance.GetProperty()))
                        if iToValue:
                                pProperty.SetCondition(iToValue)
                        elif iIncC:
                                # break here if we have done enough for this turn
                                if iNumConstructs >= self.iNumParallelConstructions:
                                        break
                                
                                iCurCondition = pProperty.GetCondition()
                                iNewCondition = iCurCondition + iIncC
                        
                                # count number of builds
                                if iCurCondition != pProperty.GetMaxCondition():
                                        iNumConstructs = iNumConstructs + 1
                                        
                                if iCurCondition + iIncC > pProperty.GetMaxCondition():
                                        iNewCondition = pProperty.GetMaxCondition()
                                pProperty.SetCondition(iNewCondition)
                
                pShipSubSystemPropInstanceList.TGDoneIterating()
                
                # if iNumConstructs is zero, then we are done with this ship
                if iIncC and iNumConstructs == 0:
                        self.iCurShipStatus = 2

        def ShipLeaveDock(self):
                # enable Glow
                if App.g_kLODModelManager.AreGlowMapsEnabled() == 1:
                        App.g_kLODModelManager.SetGlowMapsEnabled(0)
                        App.g_kLODModelManager.SetGlowMapsEnabled(1)
                
                # unset stationary
                self.GetBuildShip().GetShipProperty().SetStationary(0)
                # add it to the group
                addShipToGroup(self.GetBuildShip().GetName(), getGroupFromShip(self.GetConstructingShip().GetName()))
                # set its AI
                self.GetBuildShip().SetAI(AI.Player.FlyForward.CreatePlain(self.GetBuildShip(), 0.1))

        def ShipFinished(self):
                autoAI(self.GetBuildShip())
                self.iBuildShipID = 0
                if self.GetConstructingShip():
                        self.GetConstructingShip().EnableCollisionsWith(self.GetBuildShip(), 1)
